@@ -44,7 +44,15 @@ class TestFilePpm(PillowTestCase):
         self.assertRaises(ValueError, lambda: Image.open(path))
 
 
+    def test_neg_ppm(self):
+        # Storage.c accepted negative values for xsize, ysize.  the
+        # internal open_ppm function didn't check for sanity but it
+        # has been removed. The default opener doesn't accept negative
+        # sizes. 
+        
+        with self.assertRaises(IOError):
+            Image.open('Tests/images/negative_size.ppm')
+
+
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
