@@ -1,5 +1,6 @@
 from __future__ import print_function
 import logging
+from io import BytesIO
 import struct
 
 from helper import unittest, PillowTestCase, hopper, py3
@@ -39,7 +40,7 @@ class TestFileTiff(PillowTestCase):
         im = Image.open(filename)
 
     def test_mac_tiff(self):
-        # Read RGBa images from Mac OS X [@PIL136]
+        # Read RGBa images from macOS [@PIL136]
 
         filename = "Tests/images/pil136.tiff"
         im = Image.open(filename)
@@ -103,6 +104,15 @@ class TestFileTiff(PillowTestCase):
         im._setup()
         self.assertEqual(im.info['dpi'], (71., 71.))
 
+    def test_save_setting_missing_resolution(self):
+        from PIL.TiffImagePlugin import X_RESOLUTION, Y_RESOLUTION
+        b = BytesIO()
+        Image.open("Tests/images/10ct_32bit_128.tiff").save(
+            b, format="tiff", resolution=123.45)
+        im = Image.open(b)
+        self.assertEqual(float(im.tag_v2[X_RESOLUTION]), 123.45)
+        self.assertEqual(float(im.tag_v2[Y_RESOLUTION]), 123.45)
+
     def test_invalid_file(self):
         invalid_file = "Tests/images/flower.jpg"
 
@@ -253,6 +263,8 @@ class TestFileTiff(PillowTestCase):
 
         self.assert_warning(DeprecationWarning, im.tag_v2.as_dict)
         self.assert_warning(DeprecationWarning, im.tag.as_dict)
+        self.assertEqual(dict(im.tag_v2), im.tag_v2.as_dict())
+        self.assertEqual(dict(im.tag), im.tag.as_dict())
 
     def test_dict(self):
         # Arrange
@@ -264,7 +276,6 @@ class TestFileTiff(PillowTestCase):
                    262: 2, 296: 2, 273: (8,), 338: (1,), 277: 4,
                    279: (9460,), 282: 72.0, 283: 72.0, 284: 1}
         self.assertEqual(dict(im.tag_v2), v2_tags)
-        self.assertEqual(im.tag_v2.as_dict(), v2_tags)
 
         # legacy interface
         legacy_tags = {256: (55,), 257: (43,), 258: (8, 8, 8, 8), 259: (1,),
@@ -272,7 +283,6 @@ class TestFileTiff(PillowTestCase):
                        279: (9460,), 282: ((720000, 10000),),
                        283: ((720000, 10000),), 284: (1,)}
         self.assertEqual(dict(im.tag), legacy_tags)
-        self.assertEqual(im.tag.as_dict(), legacy_tags)
 
     def test__delitem__(self):
         filename = "Tests/images/pil136.tiff"
@@ -463,15 +473,31 @@ class TestFileTiff(PillowTestCase):
         im2 = hopper()
         self.assert_image_similar(im, im2, 5)
 
-    def test_open_tiff_uint16(self):
+    def test_roundtrip_tiff_uint16(self):
         # Test an image of all '0' values
         pixel_value = 0x1234
         infile = "Tests/images/uint16_1_4660.tif"
         im = Image.open(infile)
         self.assertEqual(im.getpixel((0, 0)), pixel_value)
 
+        tmpfile = self.tempfile("temp.tif")
+        im.save(tmpfile)
+
+        reloaded = Image.open(tmpfile)
+
+        self.assert_image_equal(im, reloaded)
+
+    def test_tiff_save_all(self):
+        import io
+        import os
+
+        mp = io.BytesIO()
+        with Image.open("Tests/images/multipage.tiff") as im:
+            im.save(mp, format="tiff", save_all=True)
+
+        mp.seek(0, os.SEEK_SET)
+        with Image.open(mp) as im:
+            self.assertEqual(im.n_frames, 3)
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
