@@ -46,7 +46,7 @@ try:
 
         def test_sanity(self):
             self.assertRegexpMatches(
-                ImageFont.core.freetype2_version, "\d+\.\d+\.\d+$")
+                ImageFont.core.freetype2_version, r"\d+\.\d+\.\d+$")
 
         def test_font_properties(self):
             ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)
@@ -121,6 +121,7 @@ try:
             size = draw.textsize(txt, ttf)
             draw.text((10, 10), txt, font=ttf)
             draw.rectangle((10, 10, 10 + size[0], 10 + size[1]))
+            del draw
 
             target = 'Tests/images/rectangle_surrounding_text.png'
             target_img = Image.open(target)
@@ -159,12 +160,20 @@ try:
 
             self.assert_image_similar(im, target_img, .5)
 
+            # Test that text() can pass on additional arguments
+            # to multiline_text()
+            draw.text((0, 0), TEST_TEXT, fill=None, font=ttf, anchor=None,
+                      spacing=4, align="left")
+            draw.text((0, 0), TEST_TEXT, None, ttf, None, 4, "left")
+            del draw
+
             # Test align center and right
             for align, ext in {"center": "_center",
                                "right": "_right"}.items():
                 im = Image.new(mode='RGB', size=(300, 100))
                 draw = ImageDraw.Draw(im)
                 draw.multiline_text((0, 0), TEST_TEXT, font=ttf, align=align)
+                del draw
 
                 target = 'Tests/images/multiline_text'+ext+'.png'
                 target_img = Image.open(target)
@@ -191,6 +200,12 @@ try:
             self.assertEqual(draw.textsize(TEST_TEXT, font=ttf),
                              draw.multiline_textsize(TEST_TEXT, font=ttf))
 
+            # Test that textsize() can pass on additional arguments
+            # to multiline_textsize()
+            draw.textsize(TEST_TEXT, font=ttf, spacing=4)
+            draw.textsize(TEST_TEXT, ttf, 4)
+            del draw
+
         def test_multiline_width(self):
             ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)
             im = Image.new(mode='RGB', size=(300, 100))
@@ -199,6 +214,7 @@ try:
             self.assertEqual(draw.textsize("longest line", font=ttf)[0],
                              draw.multiline_textsize("longest line\nline",
                                                      font=ttf)[0])
+            del draw
 
         def test_multiline_spacing(self):
             ttf = ImageFont.truetype(FONT_PATH, FONT_SIZE)
@@ -206,6 +222,7 @@ try:
             im = Image.new(mode='RGB', size=(300, 100))
             draw = ImageDraw.Draw(im)
             draw.multiline_text((0, 0), TEST_TEXT, font=ttf, spacing=10)
+            del draw
 
             target = 'Tests/images/multiline_text_spacing.png'
             target_img = Image.open(target)
@@ -229,6 +246,7 @@ try:
             # Rotated font
             draw.font = transposed_font
             box_size_b = draw.textsize(word)
+            del draw
 
             # Check (w,h) of box a is (h,w) of box b
             self.assertEqual(box_size_a[0], box_size_b[1])
@@ -251,6 +269,7 @@ try:
             # Rotated font
             draw.font = transposed_font
             box_size_b = draw.textsize(word)
+            del draw
 
             # Check boxes a and b are same size
             self.assertEqual(box_size_a, box_size_b)
@@ -346,6 +365,7 @@ try:
             # Act
             default_font = ImageFont.load_default()
             draw.text((10, 10), txt, font=default_font)
+            del draw
 
             # Assert
             self.assert_image_equal(im, target_img)
@@ -402,7 +422,7 @@ try:
 
         @unittest.skipIf(sys.platform.startswith('win32'),
                          "requires Unix or MacOS")
-        def test_find_osx_font(self):
+        def test_find_macos_font(self):
             # Like the linux test, more cover hitting code rather than testing
             # correctness.
             font_directory = '/System/Library/Fonts'
@@ -450,5 +470,3 @@ except ImportError:
 
 if __name__ == '__main__':
     unittest.main()
-
-# End of file
